# Copyright 2008 Dustin Long (dlong@progmatism.com)
#
# This file is part of Paragrasp.
#
# Paragrasp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Paragrasp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Paragrasp.  If not, see <http://www.gnu.org/licenses/>.
#
#
# A build action to create manifest files. Replaces $[var] with value.

import hashlib
import os
import StringIO
import sys


def read_config(fp):
  config = {}
  key = None
  value = None
  partial = None
  for line in fp.read().split('\n'):
    if not line or line[0] == '#':
      continue
    if partial:
      value = partial + line.strip()
      partial = None
    else:
      pos = line.find(':')
      key = line[:pos]
      value = line[pos+1:].strip()
      if value[-1] == '\\':
        partial = value[:-1]
        continue
    config[key] = value
  return config


def enhance_config(config, cmdline):
  config['updatable'] = True
  if not cmdline.param:
    return
  if cmdline.param == 'jar':
    config['protocol'] = 'jar:'
    config['container'] = config['name'] + '.jar!'
  elif cmdline.param == 'unpacked':
    config['protocol'] = ''
    config['container'] = config['name']
  elif cmdline.param == 'no_update':
    config['protocol'] = 'jar:'
    config['container'] = config['name'] + '.jar!'
    config['updatable'] = False
  elif cmdline.param == 'hash':
    xpi_file = cmdline.args[2]
    h = hashlib.sha1()
    fp = open(xpi_file, 'rb')
    h.update(fp.read())
    fp.close()
    config['hash'] = h.hexdigest()
  else:
    raise RuntimeError('Unknown param value %s' % cmdline.param)


def evaluate_variable(config, variable, status):
  variable = variable[2:-1]
  pos = variable.find(' ')
  if pos != -1:
    var_name = variable[0:pos]
    operator = variable[pos+1:]
    if operator == 'begin':
      status['output_text'] = config[var_name]
      return ''
    elif operator == 'end':
      status['output_text'] = True
      return ''
    else:
      raise RuntimeError('Unknown operator during: "%s"' % (variable,))
  return config[variable]

def process(fout, fin_config, fin_template, cmdline):
  status = {}
  status['output_text'] = True
  config = read_config(fin_config)
  enhance_config(config, cmdline)
  template_text = fin_template.read()
  start = 0
  accum_text = ''
  while True:
    pos_begin = template_text.find('$[', start)
    if pos_begin == -1:
      break
    pos_end = template_text.find(']', pos_begin + 1)
    before_text = template_text[start:pos_begin]
    if not status['output_text']:
      before_text = ''
    variable = template_text[pos_begin:pos_end + 1]
    start = pos_end + 1
    eval_text = evaluate_variable(config, variable, status)
    accum_text += before_text
    if status['output_text']:
      accum_text += eval_text
  if status['output_text']:
    accum_text += template_text[start:]
  fout.write(accum_text)


class CommandLineConfig(object):
  def __init__(self):
    self.args = []
    self.out = None
    self.param = None


def get_cmdline():
  cmdline = CommandLineConfig()
  i = 1
  while i < len(sys.argv):
    if sys.argv[i] == '-o':
      cmdline.out = sys.argv[i + 1]
      i += 2
    elif sys.argv[i] == '-p':
      cmdline.param = sys.argv[i + 1]
      i += 2
    else:
      cmdline.args.append(sys.argv[i])
      i += 1
  if len(cmdline.args) < 2 or not cmdline.out:
    sys.stderr.write('Usage: create_manifest_action.py '
                     '<config> <template> -o <out>\n')
    sys.exit(1)
  return cmdline


def run():
  cmdline = get_cmdline()
  sout = StringIO.StringIO()
  fin_config = open(cmdline.args[0], 'r')
  fin_template = open(cmdline.args[1], 'r')
  process(sout, fin_config, fin_template, cmdline)
  fin_template.close()
  fin_config.close()
  fout = open(cmdline.out, 'w')
  fout.write(sout.getvalue())
  fout.close()


if __name__ == '__main__':
  run()
