import hashlib
import shutil
import os
import re
import sys


def find_mccoy_data_dir(app_dir):
  profile_dir = os.path.join(app_dir, 'McCoy/Profiles')
  if not os.path.isdir(profile_dir):
    raise RuntimeError('Could not find mccoy profile dir\n'
                       '(Try running mccoy once to create your profile dir)')
  dir_contents = os.listdir(profile_dir)
  for entry in dir_contents:
    match = re.search('\.default$', entry)
    if match:
      return os.path.join(profile_dir, entry)
  raise RuntimeError('Could not find mccoy data dir\n'
                     '(Try running mccoy once to create your data dir)')


def hash_file(filename):
  md5 = hashlib.md5()
  fp = open(filename, 'r')
  while True:
    data = fp.read(2**20)
    if not data:
      break
    md5.update(data)
  fp.close()
  return md5.hexdigest()


def memoize_load(update_rdf, data_dir):
  md5_hash = hash_file(update_rdf)
  memo_file = os.path.join(data_dir, '.memo', md5_hash)
  fp = None
  try:
    fp = open(memo_file, 'r')
  except:
    return None
  signature = fp.read()
  fp.close()
  return signature


def memoize_save(update_rdf, data_dir, signature):
  md5_hash = hash_file(update_rdf)
  memo_file = os.path.join(data_dir, '.memo', md5_hash)
  try:
    os.mkdir(os.path.dirname(memo_file))
  except:
    pass
  fp = open(memo_file, 'w')
  fp.write(signature)
  fp.close()


def run_mccoy(update_rdf, key_database, data_dir, app, target):
  # Copy the key database to McCoy's data directory.
  shutil.copy2(key_database, os.path.join(data_dir, 'key3.db'))
  # Copy the update-unsigned.rdf to a place where McCoy will see it.
  shutil.copy2(update_rdf, target)
  print 'Running McCoy...'
  os.system(app)


def read_signature(target):
  fp = open(target, 'r')
  lines = fp.read().split('\n')
  fp.close()
  for line in lines:
    match = re.search(r'em:signature="(.*)"', line)
    if match:
      return match.group(1)
  raise RuntimeError('Signature not found!')


def clean_up(target):
  os.remove(target)


def create_signature(update_rdf, key_database):
  if sys.platform == 'darwin':
    target = os.path.join(os.environ['HOME'], 'Documents/update.rdf')
    app = '/Applications/McCoy.app/Contents/MacOS/mccoy'
    data_dir = find_mccoy_data_dir(os.path.join(os.environ['HOME'],
                                                'Library/Application Support/'))
  elif sys.platform == 'win32':
    target = os.path.join(os.getcwd(), 'update.rdf')
    app = 'mccoy.exe'
    data_dir = find_mccoy_data_dir(os.path.join(os.environ['APPDATA'],
                                                'Mozilla/'))
  else:
    raise RuntimeError('Cannot start McCoy: unknown platform')
  signature = memoize_load(update_rdf, data_dir)
  if not signature:
    run_mccoy(update_rdf, key_database, data_dir, app, target)
    signature = read_signature(target)
    clean_up(target)
    memoize_save(update_rdf, data_dir, signature)
  return signature


class CommandLineConfig(object):
  def __init__(self):
    self.args = []
    self.out = None
    self.param = None


def produce_update_file(input_rdf, output, signature):
  fin = open(input_rdf, 'r')
  fout = open(output, 'w')
  fout.write("""<?php
header("Content-type: text/xml");
print("<?xml version=\\"1.0\\"?>\\n");
?>
""")
  while True:
    line = fin.readline()
    if line.startswith('<?xml'):
      continue
    if not line:
      break
    match = re.match(r'( *)<em:updates', line)
    if match:
      spaces = match.group(1)
      fout.write("%s<em:signature>%s</em:signature>\n" % (spaces, signature))
    fout.write(line)


def get_cmdline():
  cmdline = CommandLineConfig()
  i = 1
  while i < len(sys.argv):
    if sys.argv[i] == '-o':
      cmdline.out = sys.argv[i + 1]
      i += 2
    else:
      cmdline.args.append(sys.argv[i])
      i += 1
  if len(cmdline.args) < 1 or not cmdline.out:
    sys.stderr.write('Usage: sign_with_mccoy.py <rdf> -o <out>\n')
    sys.exit(1)
  return cmdline


def run():
  cmdline = get_cmdline()
  update_rdf = cmdline.args[0]
  key_database = cmdline.args[1]
  signature = create_signature(update_rdf, key_database)
  produce_update_file(update_rdf, cmdline.out, signature)


if __name__ == '__main__':
  run()
